#include "cs.h"
#include "text.h"
#include <iostream.h>
#include <string.h>	// strchr()

void
readline( istream& is, string& s )
{
    is >> s;
}

void
dumpRes( const CSResource* r )
{
    cout << "unknown\t";
    if( r->IsLocked() )
	cout << "yes\t";
     else 
	 cout << "no\t";
    cout << r->GetName();
}

void dumpTextRes( const Text* r )
{
    cout << "PDPText\t";
    if( r->IsLocked() )
	cout << "yes\t";
     else 
	 cout << "no\t";

    cout << r->GetName() << "\ttext == \"" << r->GetText() << "\"\n";
}

void
dumpResList( Client& c )
{
    cout << "\n\n\nCurrent Resources:\nNO\tTYPE\tLOCKED\tNAME\tDATA\n";
    unsigned i;
    for( i = 0; i < c.ResCount(); ++i ) {
	cout << i << '\t';
	if( c[i]->GetType() == 1 )
	    dumpTextRes( (Text*) c[i] );
	 else
	     dumpRes( c[i] );
    }
}

char
mainMenu ()
{
    char choice;
    
    cout << "\n\nCLIENT main menu:\n\n (q)uit\n (a)dd\n (r)ead all\n"
	" (l)ock\n (u)nlock\n (e)dit\n (d)elete\n\n";
    do {
	cin >> choice;
    } while( !strchr( "qaluerd", choice ) );
    return choice;
}

CSResource*
queryRes( Client& c )
{
    cout << "\n\nEnter number of resource to process: ";
    unsigned i;
    cin >> i;
    if( i >= c.ResCount() ) {
	cout << "Invalid resource number. Stopping.\n";
	return NULL;
    }
     else
	 return c[i];
}    


/*

  Menu functions

*/

void
add( Client& c )
{
    string name, text;
    
    cout << "\n\nEnter name: ";
    readline( cin, name );
    cout << "Enter text: ";
    readline( cin, text );
    if( c.Add( new Text( name, text ) ) == csNAMEEXISTS )
	cout << "Error: resource name already exists!\n\n";
}

void
lock( Client& c )
{
    CSResource* r = queryRes( c );
    if( !r )
	return;

    switch( c.Lock( r->GetName() ) ) {
	case csOK	: cout << "OK.\n"; break;
	case csNOTFOUND : cout << "ERROR: specified resource doesn't exist\n"; break;
	case csLOCKERROR : cout << "ERROR: resource is already locked by someone else\n"; break;
    }
}

void
unlock( Client& c )
{
    CSResource* r = queryRes( c );
    if( !r )
	return;

    switch( c.Unlock( r->GetName() ) ) {
	case csOK	: cout << "OK.\n"; break;
	case csNOTFOUND : cout << "ERROR: specified resource doesn't exist\n"; break;
	 case csLOCKERROR : cout << "ERROR: resource is not locked by you!\n"; break;
    }
}

void
edit( Client& c )
{
    CSResource* r = queryRes( c );
    if( !r )
	return;
    if( r->GetType() != 1 ) {
	cout << "you cannot edit unknown resources!\n\n";
	return;
    }
    
    if( !r->IsLocked() ) {
	cout << "lock resource before editing! your changes would get lost!\n\n";
	return;
    }
    
    string text;
    cout << "Enter new Text for '" << r->GetName() << "' : ";
    readline( cin, text );
    ( (Text*) r )->SetText( text );
}

void
del( Client& c )
{
    CSResource* r = queryRes( c );
    if( !r )
	return;
    switch( c.Del( r->GetName() ) ) {
	case csOK	: cout << "OK.\n"; break;
	case csLOCKERROR : cout << "ERROR: resource is already locked by someone else\n"; break;
    }
}

int
main ()
{
    try {	
	Client client( "localhost", 7000);
	client.RegisterType( new Text( "", "" ) );
	client.LoadAll();

	bool running = true;
	
	while( running ) {
	    dumpResList( client );
	    switch( mainMenu() ){
		case 'q' : running = false; break;
		case 'a' : add( client ); break;
		case 'l' : lock( client ); break;
		case 'u' : unlock( client ); break;
		case 'e' : edit( client ); break;
		case 'd' : del( client ); break;
		default  : client.LoadAll(); break;
	    }
	}	     
    }
    catch( Exception& e ) {
	cout << e;
    }
    catch(...) {
	cout << "client: unhandled exception occured!";
    }
       
    return 0;
}

