/*
  except.h - basic excepetion class
   
  (c) 1997 Polarware
  
  This file is distributed under the terms and conditions of the 
  "GNU Library General Public License" (see file COPYING.LIB)
  
  Please report bugs and comments to martin@piware.de
*/

#ifndef EXCEPT_H
#define EXCEPT_H

#include <iostream.h>	// for overloaded <<
#include <string>

    /**
     * Basic exception. Contains the exception's description, location
     * and severity (class). Exceptions can be written directly to
     * streams.
     */
class Exception
{
 public:
	/**
	 * Class (or severity) of the exception. This can be: WARNING
	 * (nothing dangerous), NONFATAL (error, but program can be
	 * continued), FATAL (severe error, program can not be
	 * continued) or IPE (internal program error, program can not
	 * be continued).
	 */
    enum Class { FATAL, NONFATAL, IPE, WARNING };
    
 public:
	/**
	 * Sets the description, location and class of the
	 * exception. Derived classes should set what on their own,
	 * but leave where as a ctor parameter.
	 */
    Exception( const string& _where, const string& _what = "Generic Exception",
	       Class _class = FATAL) :
	    where( _where ), what( _what ), exclass( _class ) {}

	/// Destructor. Does nothing.
    virtual ~Exception() {}
    
	/// Returns the exception description as specified in the ctor.
    string What () const { return what; }

	/// Returns the exception location as specified in the ctor.
    string Where () const { return where; }

	/// Returns the exception class as specified in the ctor.
    Class GetClass () const { return exclass; }
    
	/// Convenient operator for direct output to streams.
    friend ostream& operator << (ostream& os, const Exception& e)
    {
	static const char classStr[][24] =
	  {"Fatal", "Nonfatal", "Internal program error", "Warning"};
	
	os << classStr[(char) e.GetClass ()] << " exception thrown in " <<
	    e.Where() << ":\n  " << e.What() << "\n\n";
	return os;
    }

	/// Assembles a presentable string from location and description.
    string Message() const
    {
	return string( "Exception thrown in " ) + Where() + ":\n     "
	    + What();
    }
    
 private:
    string where, what;
    Class exclass;
};

#endif // !defined( EXCEPT_H )
